package com.push.democlient;

import m.client.push.library.PushHandler;
import m.client.push.library.PushManager;
import m.client.push.library.common.Logger;
import m.client.push.library.common.PushConstants;
import m.client.push.library.common.PushConstantsEx;
import m.client.push.library.common.PushDefine;
import m.client.push.library.common.PushLog;
import m.client.push.library.utils.PushUtils;

import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.PowerManager;
import android.provider.Settings;

import android.widget.Toast;

import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import com.push.democlient.service.RealService;

public class MainActivity extends Activity {

	private Activity mMainActivity = this;
	private BroadcastReceiver mMainBroadcastReceiver;
	//private ProgressDialog mProgressDialog;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		//PushManager.getInstance().setMqttKeepAliveInterval(this,10);
		if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
			PowerManager pm = (PowerManager) getSystemService(Context.POWER_SERVICE);
			if (!pm.isIgnoringBatteryOptimizations(getPackageName())) {
				alertOfBatteryOptimize(MainActivity.this);

			} else {
				nextProcess();
			}
		}else{
			nextProcess();
		}
	}

	
	@Override
	protected void onResume() {
    	super.onResume();
    	
    	registerReceiver();
    }
	
	@Override
	protected void onPause() {
    	super.onPause();
    	
    	unregisterReceiver();
    }
	
	@Override
	public void onBackPressed() {
		super.onBackPressed();
		
		System.exit(0);
	}
	
	public void registerReceiver() {
		if (mMainBroadcastReceiver != null) {
			return;
		}
		
		// 화면에서 서비스 등록 결과를 받기 위한 리시버 등록 - 패키지명.ACTION_COMPLETED
		IntentFilter intentFilter = new IntentFilter();
		intentFilter.addAction(MainActivity.this.getPackageName() + PushConstantsEx.ACTION_COMPLETED);
		
		
		mMainBroadcastReceiver = new BroadcastReceiver() {
			
			@Override
			public void onReceive(Context context, Intent intent) {
				// 수신된 인텐트(결과값 응답) 의 정상 데이터 여부 체크 (푸시 타입/액션 타입) - 반드시 구현 
				if(! PushUtils.checkValidationOfCompleted(intent, context)){
					return;
				}
				
				//push 타입 판단 
				JSONObject result_obj = null; // 수신된 오브젝트
				String result_code = ""; // 결과 코드
				String result_msg = ""; // 결과 메세지
				try {
					result_obj = new JSONObject(intent.getExtras().getString(PushConstants.KEY_RESULT));
					result_code = result_obj.getString(PushConstants.KEY_RESULT_CODE);
					result_msg = result_obj.getString(PushConstants.KEY_RESULT_MSG);
				} 
				catch (JSONException e) {
					e.printStackTrace();
				}
				
				// 액션 타입
				String bundle = intent.getStringExtra(PushConstantsEx.KEY_BUNDLE);
				
				//액션에 따라 분기 (이미 서비스 등록이 완료된 상태인 경우 다음 process 이동) 
				if (bundle.equals(PushConstantsEx.COMPLETE_BUNDLE.IS_REGISTERED_SERVICE)) {
					//mProgressDialog.dismiss();
					Logger.e(result_code);
					String resultPushType = intent.getExtras().getString(PushConstants.KEY_PUSH_TYPE);
					if (resultPushType.equals(PushHandler.getInstance().getPushConfigInfo(getApplicationContext()).getPushType())) {
						if (result_code.equals(PushConstants.RESULTCODE_OK)) {
							Intent pageIntent = new Intent(mMainActivity, LoginActivity.class);
							startActivity(pageIntent);
							finish();
						}
					}
					else {
						Toast.makeText(mMainActivity, result_msg, Toast.LENGTH_LONG).show();
					}
					
				// 최초 서비스 등록이 완료된 경우 다음 process 이동 	
				} else if (bundle.equals(PushConstantsEx.COMPLETE_BUNDLE.REG_PUSHSERVICE)) { 
					
					// 등록 성공
					if (result_code.equals(PushConstants.RESULTCODE_OK)) {
						Toast.makeText(context, "등록 성공!", Toast.LENGTH_SHORT).show();
						Intent pageIntent = new Intent(mMainActivity, LoginActivity.class);
						startActivity(pageIntent);
						finish();
					}
					// 통신 에러
					else if (result_code.equals(PushConstants.RESULTCODE_HTTP_ERR)) {
						Toast.makeText(context, "[MainActivity] error code: " + result_code + " msg: " + result_msg, Toast.LENGTH_SHORT).show();
					}
					// 인증키 오류
					else if (result_code.equals(PushConstants.RESULTCODE_AUTHKEY_ERR)) {
						Toast.makeText(context, "[MainActivity] error code: " + result_code + " msg: " + result_msg, Toast.LENGTH_SHORT).show();
					}
					// 서버 응답 에러
					else if (result_code.equals(PushConstants.RESULTCODE_RESPONSE_ERR)) {
						Toast.makeText(context, "[MainActivity] error code: " + result_code + " msg: " + result_msg, Toast.LENGTH_SHORT).show();
					}
					// 라이브러리 에러 - 파싱 에러
					else if (result_code.equals(PushConstants.RESULTCODE_INTERNAL_ERR)) {
						Toast.makeText(context, "[MainActivity] error code: " + result_code + " msg: " + result_msg, Toast.LENGTH_SHORT).show();
					}
					// 기타
					else {
						Toast.makeText(context, "[MainActivity] error code: " + result_code + " msg: " + result_msg, Toast.LENGTH_SHORT).show();
						//System.exit(0);
					}
				}
			}
		};
		
		LocalBroadcastManager.getInstance(this).registerReceiver(mMainBroadcastReceiver, intentFilter);
	}
	
	public void unregisterReceiver() {
		if (mMainBroadcastReceiver != null) {
			LocalBroadcastManager.getInstance(this).unregisterReceiver(mMainBroadcastReceiver);
			mMainBroadcastReceiver = null;
		}
	}



	private void nextProcess(){
		if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
			Intent serviceIntent = new Intent(this, RealService.class);
			startService(serviceIntent);
		}

		// 네트워크 체크
		if (PushUtils.checkNetwork(getApplicationContext())) {

			PushManager.getInstance().initPushServer(getApplicationContext());
			Intent pageIntent = new Intent(mMainActivity, LoginActivity.class);
			startActivity(pageIntent);
			finish();
		}
		else {
			PushLog.e("MainActivity", "network is not connected.");
			Toast.makeText(MainActivity.this, "[MainActivity] network is not connected.", Toast.LENGTH_SHORT).show();
		}
	}


	private void alertOfBatteryOptimize(final Context context){

		if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.M ){

			String packageName = context.getPackageName();
			PowerManager pm = (PowerManager)context.getSystemService(Context.POWER_SERVICE);
			if (!pm.isIgnoringBatteryOptimizations(packageName)) {
				AlertDialog.Builder alert = new AlertDialog.Builder(context);
				alert.setPositiveButton("확인", new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						dialog.dismiss();     //닫기
						Intent intent = new Intent();
						String packageName = context.getPackageName();
						intent.setAction(Settings.ACTION_REQUEST_IGNORE_BATTERY_OPTIMIZATIONS);
						intent.setData(Uri.parse("package:" + packageName));
						//context.startActivity(intent);
						startActivityForResult(intent, 1234);

					}
				});
				alert.setMessage("안정적인 알림 서비스를 받기 위해, 배터리 최적화 제외가 필요합니다. \n"
						+ "\"배터리 최적화 적용 안 됨\" 팝업이 뜨는 경우,  \"예\" 를 선택해 주시기 바랍니다.");
				alert.show();
			}
		}
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		Logger.e("test");
		if(requestCode == 1234){
			if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
				PowerManager pm = (PowerManager) getSystemService(Context.POWER_SERVICE);

				if (!pm.isIgnoringBatteryOptimizations(getPackageName())) {
					alertOfBatteryOptimize(MainActivity.this);

				} else {
					nextProcess();
				}
			}
		}
	}
}
