package com.push.democlient;

import java.util.ArrayList;
import java.util.Arrays;

import org.json.JSONObject;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.BaseAdapter;
import android.widget.ListView;
import android.widget.TextView;

import com.push.democlient.db.DBUtils;
import com.push.democlient.db.DataBases;

public class PushListActivity extends Activity {
	
	private ArrayList<PushListInfo> mPushList = new ArrayList<PushListInfo>();

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		this.setContentView(R.layout.activity_list);
		
		Cursor cursor = DBUtils.getDbOpenHelper(this).getPushAppMassageALL();
		if (cursor != null && cursor.getCount() > 0 && cursor.moveToFirst()) {
			do {
				try {
					String jsonData = cursor.getString(cursor.getColumnIndex(DataBases.CreateDB.COL_MSGDATA));
					String ext = cursor.getString(cursor.getColumnIndex(DataBases.CreateDB.COL_EXTDATA));
					String date = cursor.getString(cursor.getColumnIndex(DataBases.CreateDB.COL_DATE));
					
					Log.e("PushListActivity", "[PushListActivity] jsonData:" + jsonData + ", ext:" + ext + ", date:" + date);

					JSONObject jsonMsg = new JSONObject(jsonData);
					String title = jsonMsg.getString("MESSAGE");
			    	
			    	ArrayList<String> params = new ArrayList<String>();

			    	if (ext != null) {
	    				String[] paramArray = ext.split("\\|");
	    				params = new ArrayList<String>(Arrays.asList(paramArray));
	    			}
	    			
	    			if (params != null && params.size() > 0x00) {
	    				String type = params.get(0x00);
	    				String message = params.get(0x01);
	    				if (type.equals("0")) { // 일반 메세지
	    					mPushList.add(0, new PushListInfo(type, title, message, "", "", "", date));
	    				}
	    				else if (type.equals("1")) { // 웹페이지, 웹동영상, 웹이미지 처리
	    					String imgUrl = (params.size() > 2)? params.get(2) : "";
	    					String contentUrl = (params.size() > 3)? params.get(3) : "";
	    					mPushList.add(0, new PushListInfo(type, title, message, "", contentUrl, imgUrl, date));
	    				}
	    			}
					
				} catch (Exception e) {
					e.printStackTrace();
				} 
			} while(cursor.moveToNext());
		}
		
		PushListAdapter adapter = new PushListAdapter(this, mPushList);
		ListView listView = (ListView)this.findViewById(R.id.listView);
		listView.setAdapter(adapter);
		listView.setOnItemClickListener(new OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView<?> parent, View v, int position, long id) {
				PushListInfo info = mPushList.get(position);
				String type = info.getType();
				if (type.equals("0")) { // 일반 메세지
					Intent intent = new Intent(PushListActivity.this, PushDetailActivity.class);
					intent.putExtra("type", type);
					intent.putExtra("title", info.getTitle());
			    	intent.putExtra("message", info.getMessage());
			    	startActivity(intent);
				}
				else if (type.equals("1")) { // 웹메세지 유형
					if (TextUtils.isEmpty(info.getUrl())) { // 단순 이미지만 존재하는 메세지
						Intent intent = new Intent(PushListActivity.this, PushDetailActivity.class);
						intent.putExtra("type", type);
						intent.putExtra("title", info.getTitle());
				    	intent.putExtra("message", info.getMessage());
				    	intent.putExtra("img", info.getImage());
				    	startActivity(intent);
					}
					else { // 컨텐츠가 있는 메세지 - 동영상/웹페이지
						Intent intent = new Intent(Intent.ACTION_VIEW, Uri.parse(info.getUrl()));
						startActivity(intent);
					}
				}
			}
		});
	}
	
	public class PushListAdapter extends BaseAdapter {
		
		private Context mContext;
		private ArrayList<PushListInfo> mList = new ArrayList<PushListInfo>();
		
		public PushListAdapter(Context context, ArrayList<PushListInfo> list) {
			mContext = context;
			mList = list;
		}
		
		@Override
		public int getCount() {
			return mList.size();
		}

		@Override
		public Object getItem(int arg0) {
			return null;
		}

		@Override
		public long getItemId(int arg0) {
			return 0;
		}

		@Override
		public View getView(int position, View v, ViewGroup viewGroup) {
			if (v == null) {
				LayoutInflater inflater = (LayoutInflater) mContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
				v = inflater.inflate(R.layout.listitem_push, null);
			}
			
			PushListInfo info = mList.get(position);
			
			TextView titleTextView = (TextView) v.findViewById(R.id.listitem_textiew_title);
			titleTextView.setText(info.getTitle());
			
			TextView messageTextView = (TextView) v.findViewById(R.id.listitem_textiew_message);
			messageTextView.setText(info.getMessage());
			
			TextView dateTextView = (TextView) v.findViewById(R.id.listitem_textiew_date);
			dateTextView.setText(info.getDate());
			
			v.getTag(position);
			
			return v;
		}
	}
	
	public class PushListInfo {
		private String type;
		private String title;
		private String message;
		private String encrypt;
		private String url;
		private String image;
		private String date;
		
		public PushListInfo(String type, String title, String message, String encrypt, String url, String image, String date) {
			this.type = type;
			this.title = title;
			this.message = message;
			this.encrypt = encrypt;
			this.url = url;
			this.image = image;
			this.date = date;
		}
		
		public String getType() {
			return type;
		}
		public String getTitle() {
			return title;
		}
		public String getMessage() {
			return message;
		}
		public String getEncrypt() {
			return encrypt;
		}
		public String getUrl() {
			return url;
		}
		public String getImage() {
			return image;
		}
		public String getDate() {
			return date;
		}
	}
}
