package com.push.democlient.notification;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Handler;
import android.os.Message;
import androidx.core.app.NotificationCompat;

import android.text.TextUtils;
import android.util.Log;
import android.view.View;

import com.nostra13.universalimageloader.core.ImageLoader;
import com.nostra13.universalimageloader.core.ImageLoaderConfiguration;
import com.nostra13.universalimageloader.core.assist.FailReason;
import com.nostra13.universalimageloader.core.listener.SimpleImageLoadingListener;
import com.push.democlient.PushMessageActivity;
import com.push.democlient.R;
import com.push.democlient.db.DBUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;

import m.client.push.library.common.PushConstants;

public class UpnsNotifyHelper {

	/**
	 * showNotification
	 * 메세지를 분석하여 타입 별 노티피케이션 생성
	 * @param context
	 * @param jsonMsg
	 * @param psid
	 * @param encryptData
	 * @throws Exception
	 */
	public static void showNotification(final Context context, final JSONObject jsonMsg, final String psid, final String encryptData) throws Exception {
		String extData = "";
		final String SEQNO = jsonMsg.getString("SEQNO");
		if (jsonMsg.has(PushConstants.KEY_EXT)) {
			extData = jsonMsg.getString(PushConstants.KEY_EXT);
		}

		// EXT가 존재할 경우 다음과 같은 형태로 전달됨
		// extData = "http:/211.241.199.158:8180/msg/totalInfo/0218115649_msp.html";
		if (extData.startsWith("http")) { // HTTP 로 시작하는 url 형태일 경우 - 메세지 구분을 위한 스트링 형태로 서버로부터 내려받는다.
			// HttpGetStringThread로부터 응답값을 받은 후 처리 로직
			Handler handler = new Handler() {
				@Override
				public void handleMessage(Message msg) {
					if(msg.what == 0) {
						String message = (String) msg.obj;
						if (message != null) {
							//message = message.replaceAll("https", "http");
							message = message.replaceAll("\\\\", "/");

							try {
								// 다음과 같은 형태로 응답 값이 구성되었을 경우 치환을 위한 로직 수행
								// "TYPE":"R","VAR":"hoseok|2015-06-31|2015-07-30|104320|11"
								if (jsonMsg.has("TYPE") && jsonMsg.getString("TYPE").equals("R")) {
									String var = jsonMsg.getString("VAR");
									HashMap<String, String> varMap = new HashMap<String, String>();
									if (var != null) {
										String[] varArray = var.split("\\|");
										for (int i=0; i<varArray.length; i++) {
											int idx = i+1;
											varMap.put("%VAR" + idx + "%", varArray[i]);
											//Log.d("test", "%VAR" + idx + "%" + " " +  varArray[i]);
										}

										Iterator<?> keys = varMap.keySet().iterator();
										while (keys.hasNext()) {
											String key = (String) keys.next();
											message = message.replaceAll(key, varMap.get(key));
											//Log.d("test", "message: " + message);
										}
									}
								}
							} catch (JSONException e) {
								e.printStackTrace();
							}
						}

						DBUtils.getDbOpenHelper(context).insertPushMSG(SEQNO, jsonMsg.toString(), message, encryptData);
						createNotification(context, jsonMsg, psid, encryptData, message);
					}
				}
			};
			// EXT url 로부터 메세지 정보를 수신하기 위한 쓰레드
			new HttpGetStringThread(handler, extData).start();
		}
		else { // 일반 스크링의 경우
			DBUtils.getDbOpenHelper(context).insertPushMSG(SEQNO, jsonMsg.toString(), extData, encryptData);
			createNotification(context, jsonMsg, psid, encryptData, extData);
		}
	}

	/**
	 * createNotification
	 * 노티피케이션 생성
	 * @param context
	 * @param jsonMsg
	 * @param psid
	 * @param encryptData
	 * @param message
	 */
	private static void createNotification(final Context context, final JSONObject jsonMsg, final String psid, final String encryptData, String message) {
		ArrayList<String> params = null;
		if (message != null) {
			String[] paramArray = message.split("\\|");
			params = new ArrayList<String>(Arrays.asList(paramArray));
		}
		try {
			jsonMsg.remove("EXT");
			jsonMsg.put("EXT", message);
		} catch (JSONException e) {
			e.printStackTrace();
		}
		if (message != null && !message.contains("|")){
			String imgUrl = "";
			try {
				JSONObject object = new JSONObject(message);
				imgUrl = object.optString("imgUrl");
				if (imgUrl != null && !imgUrl.isEmpty()) {
					Log.e("upns_message::",object.optString("imgUrl"));
					showImageNotification(context, jsonMsg, message, imgUrl, psid, message);
				}else{
					defaultNotification(context, jsonMsg, message, psid, message, encryptData, null);
				}
			} catch (JSONException e) {
				defaultNotification(context, jsonMsg, message, psid, message, encryptData, null);
			}
		} else {
			// 메세지 타입에 따라 노티피케이션 생성 - 용도에 따라 커스터마이징하여 사용이 가능 (타입 번호/파싱 룰 등등..)
			if (params != null && params.size() > 0) {
				Log.d("NotificationManager", "[NotificationManager] params size:: " + params.size());
				try {
					// URL을 실제 상세 메세지 데이터로 치환
					jsonMsg.remove(PushConstants.KEY_EXT);
					jsonMsg.put(PushConstants.KEY_EXT, message);

					// 일반 메세지의 경우
					if (params.get(0).equals("0") || params.get(0).equals("4") || params.get(0).equals("6")) {
						Log.d("NotificationManager", "[NotificationManager] defaultNotification: " + message);
						defaultNotification(context, jsonMsg, params.get(1), psid, message, encryptData, null);
					} else if (params.get(0).equals("11")) { //시큐어 푸시를 위한 구분 (Icon 이미지를 포함한 ext)
						Log.d("NotificationManager", "[NotificationManager] defaultNotification(11): " + message);
						showIconNotification(context, jsonMsg, params.get(1), psid, message, encryptData, params.get(2));
					}
					// 이미지 메세지의 경우
					else if (params.size() > 2) {
						Log.d("NotificationManager", "[NotificationManager] showImageNotification: " + message);
						String url = params.get(2);
						if (TextUtils.isEmpty(url) || "null".equals(url)) {
							defaultNotification(context, jsonMsg, params.get(1), psid, message, encryptData, null);
						} else {
							showImageNotification(context, jsonMsg, params.get(1), params.get(2), psid, message);
						}
					}
					// 기타의 경우 - 일반메세지로 처리했으나 용도에 맞게 변경
					else {
						Log.d("NotificationManager", "[NotificationManager] UNKNOUWN TYPE:: " + params.get(0));
						defaultNotification(context, jsonMsg, message, psid, message, encryptData, null);
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
	}
	private static void showIconNotification(final Context context, final JSONObject jsonMsg,
											 final String strMessage, final String psid, final String ext, final String encryptData, String iconPath) {

		//iconPath = "http://211.241.199.216:28080/secure-push-gw/ic_default.png";
		if (!iconPath.startsWith("http://")) {
			iconPath = iconPath.replaceAll("http:/", "http://");
		}
		if (!ImageLoader.getInstance().isInited()) {
			ImageLoaderConfiguration config = new ImageLoaderConfiguration.Builder(context).build();
			ImageLoader.getInstance().init(config);
		}
		ImageLoader.getInstance().loadImage(iconPath, new SimpleImageLoadingListener() {
			@Override
			public void onLoadingFailed(String imageUri, View view, FailReason failReason) {
				defaultNotification(context, jsonMsg, strMessage, psid, ext, encryptData, null);
			}

			@Override
			public void onLoadingComplete(String imageUri, View view, Bitmap loadedImage) {
				System.out.println("loadedImage.getWidth(): " + loadedImage.getWidth());
				System.out.println("loadedImage.getHeight(): " + loadedImage.getHeight());
				Bitmap scaledImage = Bitmap.createScaledBitmap(loadedImage, 192, 192, false);
				if (loadedImage != null) {
					loadedImage.recycle();
					loadedImage = null;
				}
				defaultNotification(context, jsonMsg, strMessage, psid, ext, encryptData, scaledImage);
			}
		});
	}

	public static String NOTIFICATION_CHANNEL_ID = "";
	public static CharSequence name = "";
	private static void defaultNotification(Context context, JSONObject jsonMsg, String strMessage, String psid, String ext, String encryptData, Bitmap loadedIcon) {
		NOTIFICATION_CHANNEL_ID = context.getPackageName() + "_1";
		String alertMessage = "";
		String title = context.getString(R.string.app_name);
		long seqno = 0;
		String seq;
		int icon = R.drawable.ic_launcher;

		try {
			alertMessage = jsonMsg.optString(PushConstants.KEY_MESSAGE);
			seq = jsonMsg.optString(PushConstants.KEY_SEQNO);
			title = getAppName(context);
			if (!seq.isEmpty()) {
				seqno = Long.parseLong(seq);
			}
		} catch (Exception e1) {
			e1.printStackTrace();
		}
		int hashedId = (int) (seqno ^ (seqno >>> 32)); // long 값을 안전하게 변환

		Intent intent = new Intent(context, PushMessageActivity.class);
		intent.putExtra(PushConstants.KEY_JSON, jsonMsg.toString());
		intent.putExtra(PushConstants.KEY_PSID, psid);
		intent.putExtra(PushConstants.KEY_TITLE, alertMessage);
		intent.putExtra(PushConstants.KEY_EXT, ext);
		intent.putExtra("ENCRYPT", encryptData);
		intent.putExtra(PushConstants.KEY_PUSHTYPE, "UPNS");
		intent.addFlags(Intent.FLAG_ACTIVITY_NO_HISTORY);

		NotificationManager mManager = (NotificationManager)context.getSystemService(Context.NOTIFICATION_SERVICE);
		name = getAppName(context);
		PendingIntent pIntent = PendingIntent.getActivity(context, hashedId, intent, PendingIntent.FLAG_UPDATE_CURRENT | PendingIntent.FLAG_IMMUTABLE);

		if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
			int importance = NotificationManager.IMPORTANCE_HIGH;
			NotificationChannel mChannel = new NotificationChannel(NOTIFICATION_CHANNEL_ID, name, importance);
			mChannel.setDescription(context.getPackageName());
			mChannel.enableLights(true);
			mChannel.setLightColor(Color.RED);
			mChannel.enableVibration(true);
			mChannel.setVibrationPattern(new long[]{100, 200, 300, 400, 500, 400, 300, 200, 400});
			mManager.createNotificationChannel(mChannel);
		}

		Uri soundUri = RingtoneManager.getDefaultUri(RingtoneManager.TYPE_NOTIFICATION);

		NotificationCompat.Builder mBuilder = new NotificationCompat.Builder(context.getApplicationContext(), NOTIFICATION_CHANNEL_ID)
				.setAutoCancel(true)
				.setContentIntent(pIntent)
				.setSmallIcon(icon)
				// .setLargeIcon(BitmapFactory.decodeResource(context.getResources(), icon))
				.setContentTitle(title)
				.setContentText(alertMessage)
				.setTicker(title)
				.setSound(soundUri);
		if (icon != 0){
			mBuilder.setSmallIcon(icon);
		}
		final Notification notify = mBuilder.build();
		mManager.notify("upns", hashedId, notify);
	}

	private static void showImageNotification(Context context, final JSONObject jsonMsg, final String strMessage, String img, final String psid, final String ext) {
		final int icon = R.drawable.ic_launcher;
		final String title = getAppName(context);
		final Context ctx = context;

		if (img.contains("https")) {
			//img = img.replaceAll("https", "http");
			img = img.replaceAll("\\\\", "/");
		}

		ImageLoaderConfiguration config = new ImageLoaderConfiguration.Builder(context).build();
		ImageLoader.getInstance().init(config);
		ImageLoader.getInstance().loadImage(img, new SimpleImageLoadingListener() {

			@Override
			public void onLoadingComplete(String imageUri, View view, Bitmap loadedImage) {
				notificationWithBigPicture(ctx, jsonMsg, title, strMessage, icon, loadedImage, psid, ext);
			}
		});
	}

	private static void notificationWithBigPicture(Context context, JSONObject jsonMsg, String title, String message, int icon, Bitmap banner, String psid, String ext) {

		Intent intent = new Intent(context, PushMessageActivity.class);
		intent.putExtra(PushConstants.KEY_JSON, jsonMsg.toString());
		intent.putExtra(PushConstants.KEY_PSID, psid);
		intent.putExtra(PushConstants.KEY_TITLE, message);
		intent.putExtra(PushConstants.KEY_EXT, ext);
		intent.putExtra(PushConstants.KEY_PUSHTYPE, "UPNS");
		intent.addFlags(Intent.FLAG_ACTIVITY_NO_HISTORY);
		long seqno = 0;

		NOTIFICATION_CHANNEL_ID = context.getPackageName() +"_1";
		name = getAppName(context);

		try {
			seqno = Long.parseLong(jsonMsg.getString("SEQNO"));
		} catch (NumberFormatException | JSONException e) {
			e.printStackTrace();
		}

		NotificationManager mManager = (NotificationManager)context.getSystemService(Context.NOTIFICATION_SERVICE);
		if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
			int importance = NotificationManager.IMPORTANCE_HIGH;
			String Description = context.getPackageName();
			NotificationChannel mChannel = new NotificationChannel(NOTIFICATION_CHANNEL_ID, name, importance);
			mChannel.setDescription(Description);
			mChannel.enableLights(true);
			mChannel.setLightColor(Color.RED);
			mChannel.enableVibration(true);
			mChannel.setVibrationPattern(new long[]{100, 200, 300, 400, 500, 400, 300, 200, 400});
			mManager.createNotificationChannel(mChannel);

		}
		int hashedId = (int) (seqno ^ (seqno >>> 32));
		PendingIntent pendingIntent = PendingIntent.getActivity(context, hashedId, intent, PendingIntent.FLAG_UPDATE_CURRENT | PendingIntent.FLAG_IMMUTABLE);
		NotificationCompat.Builder builder = new NotificationCompat.Builder(context, NOTIFICATION_CHANNEL_ID)
				.setSmallIcon(icon)
				.setTicker(title)
				.setContentText(message)
				.setPriority(Notification.PRIORITY_HIGH)
				.setContentText(message)
				.setAutoCancel(true);

		NotificationCompat.BigPictureStyle style = new NotificationCompat.BigPictureStyle();
		style.setBigContentTitle(title);
		style.setSummaryText(message);
		style.bigPicture(banner);

		builder.setStyle(style);
		builder.setContentIntent(pendingIntent);

		builder.setDefaults(Notification.DEFAULT_VIBRATE);
		builder.setSound(RingtoneManager.getDefaultUri(RingtoneManager.TYPE_NOTIFICATION));
		mManager.notify("upns", hashedId/*seqno*/, builder.build());
	}

	public static String getAppName(Context context) {
		String appName = "";
		try {
			PackageManager pm = context.getPackageManager();
			PackageInfo i = pm.getPackageInfo(context.getPackageName(), 0);
			appName = i.applicationInfo.loadLabel(pm) + "";
		} catch(PackageManager.NameNotFoundException e) { }
		return appName;
	}
}