package com.push.democlient.service;

import android.annotation.TargetApi;
import android.app.AlarmManager;
import android.app.Application;
import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.os.Handler;
import android.os.IBinder;
//import android.support.v4.app.NotificationCompat;
import android.os.Looper;
import android.util.Log;
import android.widget.Toast;

import androidx.core.app.NotificationCompat;

import com.push.democlient.MainActivity;
import com.push.democlient.R;

import java.util.Calendar;

import m.client.push.library.common.Logger;
import m.client.push.library.network.PushConnectivityManager;
import m.client.push.library.service.UPNSJobService;

public class RealService extends Service {

    public static Intent serviceIntent = null;
    public Context context;

    public RealService() {
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        serviceIntent = intent;
        showToast(getApplication(), "Start Service");

        NotificationCompat.Builder builder = new NotificationCompat.Builder(this, "default");
        builder.setSmallIcon(R.drawable.ic_launcher);

        builder.setContentTitle(getString(R.string.app_name));
        builder.setContentText("알림 서비스가 실행중입니다.");
        Intent notificationIntent = new Intent(this, MainActivity.class);
        PendingIntent pendingIntent = PendingIntent.getActivity(this, 0, notificationIntent, PendingIntent.FLAG_UPDATE_CURRENT | PendingIntent.FLAG_IMMUTABLE);
        builder.setContentIntent(pendingIntent);
        builder.setPriority(Notification.PRIORITY_MIN);

        NotificationManager manager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            manager.createNotificationChannel(new NotificationChannel("default", "기본 채널", NotificationManager.IMPORTANCE_MIN));
        }

        Notification notification = builder.build();
        startForeground(9, notification);
        Logger.e("fore ground command");

        UPNSJobService.getInstance().actionReconnect(this);

        return START_STICKY;
    }
    public boolean isNetworkAvailable() {
        PushConnectivityManager mConnManager = new PushConnectivityManager(this);
        return mConnManager.isConnected(this);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();

        serviceIntent = null;
        setAlarmTimer();



    }

    @Override
    public void onCreate() {
        super.onCreate();
    }

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    @Override
    public boolean onUnbind(Intent intent) {
        return super.onUnbind(intent);
    }

    public void showToast(final Application application, final String msg) {
        Handler h = new Handler(application.getMainLooper());
        h.post(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(application, msg, Toast.LENGTH_LONG).show();
            }
        });
    }

    protected void setAlarmTimer() {
        Logger.i("alarm restart");
        final Calendar c = Calendar.getInstance();
        c.setTimeInMillis(System.currentTimeMillis());
        c.add(Calendar.SECOND, 1);
        Intent intent = new Intent(this, AlarmRecever.class);
        PendingIntent sender = PendingIntent.getBroadcast(this, 0,intent,PendingIntent.FLAG_UPDATE_CURRENT | PendingIntent.FLAG_IMMUTABLE);

        AlarmManager mAlarmManager = (AlarmManager) getSystemService(Context.ALARM_SERVICE);
        mAlarmManager.set(AlarmManager.RTC_WAKEUP, c.getTimeInMillis(), sender);
    }

    private void foreGroundStop(){
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            stopForeground(true);
            stopForeground(Service.STOP_FOREGROUND_REMOVE);
            stopSelf();

        } else {
            stopSelf();
        }
    }

    @TargetApi(35)
    @Override
    public void onTimeout(int startId, int fgsType) {
        Log.e("ForegroundService", "Service timeout detected. Attempting to restart.");
        foreGroundStop();
        // 재시작할 Intent 생성
        Intent restartIntent = new Intent(this, RealService.class);
        restartIntent.putExtra("fgsType", fgsType); // 포그라운드 서비스 유형 전달

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            Log.d("ForegroundService", "Restarting service with startForegroundService.");
            startForegroundService(restartIntent); // Android 8.0 이상
        } else {
            Log.d("ForegroundService", "Restarting service with startService.");
            startService(restartIntent); // Android 7.1 이하
        }
    }

}